
#include <math.h>
#include <cmath>

#include <stdio.h>
#include <omp.h>
#include <vector>
#include <algorithm>
#include <string.h>
#include <iostream>

using namespace std;

extern "C" {
#include "iio.h"
}


/**
 *
 *  Function to read images using the iio library
 *  It allocates memory for the image and returns true if it
 *  correctly reads the image.
 *
 */
bool read_image(const char *fname, float **f, int &nx, int &ny, int &nz){
	
	*f = iio_read_image_float_vec(fname, &nx, &ny, &nz);
	
	return *f ? true : false;
}



int main(int argc, char *argv[])
{
	if(argc < 2){
		cout << "Usage: " << argv[0] << " Input" << endl;
		return 0;
	}	

	int i = 1, nx, ny, nz;
	
	const char *input  = argv[i++]; 
		
	float *I;
	bool correct1 = read_image (input, &I, nx, ny, nz);
    
    if (!correct1){
		std::cout << "Problem reading the input image" << std::endl;
		return 1;
    }
    
	if(nz > 0){
		
		float *R  = new float[nx * ny * nz];
		float *G  = new float[nx * ny * nz];
		float *B  = new float[nx * ny * nz];

		int dg=1, db=2;
		if(nz == 1) dg=db=0;
		
	   	for(int i = 0; i < ny; i++)
		for(int j = 0; j < nx; j++)
		{
			R[(i*nx+j)*nz] = I[(i*nx+j)*nz];
			R[(i*nx+j)*nz+dg] = 0;
		    R[(i*nx+j)*nz+db] = 0;
			
			G[(i*nx+j)*nz] = 0;
			G[(i*nx+j)*nz+dg] = I[(i*nx+j)*nz+dg];
		    G[(i*nx+j)*nz+db] = 0;
			
			B[(i*nx+j)*nz] = 0;
			B[(i*nx+j)*nz+dg] = 0;
		    B[(i*nx+j)*nz+db] = I[(i*nx+j)*nz+db];
	    
	    }
		iio_save_image_float_vec("R.png", R, nx, ny, nz);
		iio_save_image_float_vec("G.png", G, nx, ny, nz);
		iio_save_image_float_vec("B.png", B, nx, ny, nz);
		
		/*
		float *R  = new float[nx * ny];
		float *G  = new float[nx * ny];
		float *B  = new float[nx * ny];

		int dg=1, db=2;
		if(nz == 1) dg=db=0;
		 
		for(int i = 0; i < ny; i++)
		for(int j = 0; j < nx; j++)
		{
			R[(i*nx+j)] = I[(i*nx+j)];
			G[(i*nx+j)] = I[(i*nx+j)*nz+dg];
		    B[(i*nx+j)] = I[(i*nx+j)*nz+db];
	    
	    }
		iio_save_image_float_vec("R.png", R, nx, ny, 1);
		iio_save_image_float_vec("G.png", G, nx, ny, 1);
		iio_save_image_float_vec("B.png", B, nx, ny, 1);
		*/
		
				
		delete[]R;
		delete[]G;
		delete[]B;
	}

	free(I);
    return 0;
}
